package com.io.yutian.elementoriginlib;

import com.io.yutian.elementoriginlib.config.MongodbConfig;
import com.io.yutian.elementoriginlib.config.OriginLibConfig;
import com.io.yutian.elementoriginlib.config.RedisConfig;
import com.io.yutian.elementoriginlib.lang.Lang;
import com.io.yutian.elementoriginlib.listener.GuiHandlerListener;
import com.io.yutian.elementoriginlib.listener.PlayerChatInputListener;
import com.io.yutian.elementoriginlib.logger.Logger;
import com.io.yutian.elementoriginlib.manager.CommandManager;
import com.io.yutian.elementoriginlib.redis.RedisIO;
import com.io.yutian.elementoriginlib.util.FileUtil;
import com.io.yutian.elementoriginlib.ztsd.ZstdDictionaryManager;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.io.IOException;
import java.time.Duration;

public final class ElementOriginLib extends JavaPlugin {

    public static final Logger LOGGER = Logger.getLogger(ElementOriginLib.class);

    private static ElementOriginLib instance;

    private RedisIO redisIO;

    private OriginLibConfig config;
    private RedisConfig redisConfig;
    private MongodbConfig mongodbConfig;
    private ZstdDictionaryManager zstdDictionaryManager;

    @Override
    public void onEnable() {
        instance = this;

        new GuiHandlerListener(this);
        new PlayerChatInputListener(this);

        CommandManager.INSTANCE.registerPluginCommand("elementoriginlib");

        reload();

        redisIO = new RedisIO();
        redisIO.init(redisConfig);

        ZstdDictionaryManager.DictionaryConfig config = new ZstdDictionaryManager.DictionaryConfig.Builder()
                .minSampleSize(1024)                    // 最小样本大小：1KB
                .maxSampleSize(16 * 1024 * 1024)        // 最大样本大小：16MB
                .minQualityThreshold(0.5)               // 最小样本质量阈值
                .compressionLevel(3)                    // 压缩级别（1-22，3是平衡值）
                .maxDictionarySize(128 * 1024)           // 字典大小：128B
                .trainingInterval(Duration.ofHours(1)) // 训练间隔：1小时
                .maxSamples(1024)                        // 触发训练的样本数量
                .maxSampleFileSize(512 * 1024 * 1024)    // 样本文件大小限制：512MB
                .build();

        try {
            zstdDictionaryManager = new ZstdDictionaryManager("plugins/pixeldatasync/zstd_dictionary", config);
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    @Override
    public void onDisable() {
        redisIO.close();
        zstdDictionaryManager.shutdown();
    }

    public void reload() {
        config = new OriginLibConfig();
        saveDefaultConfig();
        reloadConfig();
        redisConfig = new RedisConfig();
        File redisFile = new File(getDataFolder(), "redis.yml");
        if (!redisFile.exists()) {
            saveResource("redis.yml", false);
        }
        FileConfiguration redisFileConfig = YamlConfiguration.loadConfiguration(redisFile);
        redisConfig.load(redisFileConfig);
        mongodbConfig = new MongodbConfig();
        File mongodbFile = new File(getDataFolder(), "mongodb.yml");
        if (!mongodbFile.exists()) {
            saveResource("mongodb.yml", false);
        }
        FileConfiguration mongodbFileConfig = YamlConfiguration.loadConfiguration(mongodbFile);
        mongodbConfig.load(mongodbFileConfig);
        config.load(getConfig());
        Lang.registerLangFile(this);
        Lang.reload();
    }

    public OriginLibConfig getLibConfig() {
        return config;
    }

    public RedisConfig getRedisConfig() {
        return redisConfig;
    }

    public MongodbConfig getMongodbConfig() {
        return mongodbConfig;
    }

    public RedisIO getRedisIO() {
        return redisIO;
    }

    public static ElementOriginLib inst() {
        return instance;
    }

}
