package com.io.yutian.elementoriginlib.command;

import org.bukkit.Bukkit;
import org.bukkit.World;
import org.bukkit.entity.Player;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.function.Supplier;

public class Suggests {

    public static final Suggest WORLD_LIST = ()->{
        List<String> list = new LinkedList<>();
        for (World world : Bukkit.getWorlds()) {
            list.add(world.getName());
        }
        return list;
    };

    public static final Suggest PLAYER_LIST = ()->{
        List<String> list = new LinkedList<>();
        for (Player player : Bukkit.getOnlinePlayers()) {
            list.add(player.getName());
        }
        return list;
    };

    private static final Map<String, Suggest> suggests = new HashMap<>();

    public static void registerSuggest(String name, Suggest suggest) {
        suggests.put(name, suggest);
    }

    public static Suggest getSuggest(String name) {
        return suggests.get(name);
    }

    public static void unregisterSuggest(String name) {
        suggests.remove(name);
    }

    public static Suggest createEnumSuggest(Class<? extends Enum> enumClass) {
        return new EnumSuggest(enumClass);
    }

    public static Suggest createListSuggest(Supplier<List<String>> supplier) {
        return new ListSuggest(supplier);
    }

    private static class EnumSuggest implements Suggest {

        private List<String> suggestions = new LinkedList<>();
        private Class<? extends Enum> enumClass;

        public EnumSuggest(Class<? extends Enum> enumClass) {
            this.enumClass = enumClass;
            if (enumClass != null && enumClass.isEnum()) {
                Enum<?>[] constants = enumClass.getEnumConstants();
                if (constants != null) {
                    for (Enum<?> enumConstant : constants) {
                        suggestions.add(enumConstant.name());
                    }
                }
            }
        }

        @Override
        public List<String> getSuggest() {
            return suggestions;
        }
    }

    private static class ListSuggest implements Suggest {

        private Supplier<List<String>> supplier;

        public ListSuggest(Supplier<List<String>> supplier) {
            this.supplier = supplier;
        }

        @Override
        public List<String> getSuggest() {
            return supplier.get();
        }
    }

    static {

        registerSuggest("world_list", WORLD_LIST);
        registerSuggest("player_list", PLAYER_LIST);

    }

}
