package com.io.yutian.elementoriginlib.item;

import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.ItemStats;
import com.io.yutian.elementoriginlib.item.stat.StatData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.*;

public class TagStatItem {

    private ItemStack itemStack;
    private ItemProxy itemProxy;
    private Map<ItemStat, StatData> stats = new HashMap<>();

    private TagStatItemStackBuilder builder;

    public TagStatItem(ItemStack itemStack) {
        this.itemStack = itemStack;
        this.itemProxy = new ItemProxy(itemStack);

        this.builder = new TagStatItemStackBuilder(this);
    }

    public void setStatData(@NotNull Class<? extends ItemStat> statClass, @NotNull StatData data) {
        ItemStat itemStat = ItemStats.getItemStat(statClass);
        if (itemStat == null) {
            return;
        }
        setStatData(itemStat, data);
    }

    public void setStatData(@NotNull ItemStat stat, @NotNull StatData data) {
        this.stats.put(stat, data);
        this.itemStack = builder().build();
    }

    public void removeStatData(@NotNull Class<? extends ItemStat> statClass) {
        ItemStat itemStat = ItemStats.getItemStat(statClass);
        if (itemStat == null) {
            return;
        }
        removeStatData(itemStat);
    }

    public void removeStatData(@NotNull ItemStat stat) {
        this.stats.remove(stat);
        this.itemStack = builder().build();
    }

    public <S extends StatData> S getStatData(@NotNull Class<? extends ItemStat<S>> statClass) {
        ItemStat itemStat = ItemStats.getItemStat(statClass);
        if (itemStat == null) {
            return null;
        }
        return (S) getStatData(itemStat);
    }

    public <S extends StatData> S getStatData(@NotNull ItemStat<S> stat) {
        if (!hasStatData(stat)) {
            stat.load(this);
        }
        return (S) this.stats.get(stat);
    }

    public boolean hasStatData(@NotNull Class<? extends ItemStat> statClass) {
        ItemStat itemStat = ItemStats.getItemStat(statClass);
        if (itemStat == null) {
            return false;
        }
        return hasStatData(itemStat);
    }

    public boolean hasStatData(@NotNull ItemStat stat) {
        if (!this.stats.containsKey(stat)) {
            try {
                stat.load(this);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return this.stats.containsKey(stat);
    }

    @NotNull
    public TagStatItemStackBuilder builder() {
        return this.builder;
    }

    @NotNull
    public Set<ItemStat> getStatTypes() {
        return Collections.unmodifiableSet(this.stats.keySet());
    }

    @NotNull
    public Map<ItemStat, StatData> getStats() {
        for (ItemStat stat : ItemStats.getItemStats()) {
            if (!hasStatData(stat)) {
                try {
                    stat.load(this);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return Collections.unmodifiableMap(stats);
    }

    public ItemStack getItemStack() {
        return itemStack;
    }

    public ItemProxy getItemProxy() {
        return itemProxy;
    }

    @Override
    public TagStatItem clone() {
        TagStatItem tagStatItem = new TagStatItem(itemStack.clone());
        for (ItemStat itemStat : this.stats.keySet()) {
            tagStatItem.stats.put(itemStat, this.stats.get(itemStat));
        }
        return tagStatItem;
    }

}
