package com.io.yutian.elementoriginlib.item.stat;

import com.io.yutian.elementoriginlib.item.stat.list.IdStat;
import com.io.yutian.elementoriginlib.logger.Logger;
import org.jetbrains.annotations.Nullable;

import java.util.*;

public class ItemStats {

    private static final Logger LOGGER = Logger.getLogger(ItemStats.class);

    private static Map<Class<? extends ItemStat>, ItemStat<?>> itemStats = new HashMap<>();

    static {
        register(new IdStat());
    }

    public static void register(ItemStat itemStat) {
        Class<? extends ItemStat> clazz = itemStat.getClass();
        if (isRegistered(itemStat)) {
            LOGGER.warn("ItemStat "+clazz.getName()+" 已注册，将覆盖原有注册");
            Set<Class<? extends ItemStat>> removeSet = new HashSet<>();
            for (Class<? extends ItemStat> clazz1 : itemStats.keySet()) {
                if (clazz1.getName().equalsIgnoreCase(itemStat.getClass().getName())) {
                    removeSet.add(clazz1);
                }
            }
            removeSet.forEach((aClass -> itemStats.remove(aClass)));
        }
        itemStats.put(clazz, itemStat);
    }

    private static boolean isRegistered(ItemStat itemStat) {
        for (Class<? extends ItemStat> clazz : itemStats.keySet()) {
            if (clazz.getName().equalsIgnoreCase(itemStat.getClass().getName())) {
                return true;
            }
        }
        return itemStats.containsKey(itemStat.getClass());
    }

    public static void unregister(ItemStat itemStat) {
        itemStats.remove(itemStat.getId());
    }

    @Nullable
    public static ItemStat getItemStat(Class<? extends ItemStat> clazz) {
        return itemStats.get(clazz);
    }

    @Nullable
    public static ItemStat getItemStat(String id) {
        for (ItemStat itemStat : itemStats.values()) {
            if (itemStat.getId().equalsIgnoreCase(id)) {
                return itemStat;
            }
        }
        return null;
    }

    public static Collection<ItemStat> getItemStats() {
        return Collections.unmodifiableCollection(itemStats.values());
    }

}
