package com.io.yutian.elementoriginlib.item.stat.type;

import com.io.yutian.elementoriginlib.item.TagStatItemStackBuilder;
import com.io.yutian.elementoriginlib.item.stat.ItemStat;
import com.io.yutian.elementoriginlib.item.stat.data.EnumListData;
import com.io.yutian.elementoriginlib.tag.ItemProxy;
import com.io.yutian.elementoriginlib.tag.TagList;
import com.io.yutian.elementoriginlib.tag.TagString;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;


public abstract class EnumListStat<E extends Enum> extends ItemStat<EnumListData<E>> {

    private Class<E> clazz;

    public EnumListStat(@NotNull String id, @NotNull String path, Class<E> clazz) {
        super(id, path);
        this.clazz = clazz;
    }

    @Override
    public void whenApplied(@NotNull TagStatItemStackBuilder itemStackBuilder, @NotNull EnumListData<E> statData) {
        ItemProxy itemProxy = itemStackBuilder.getItemProxy();
        TagList<TagString> tagList = new TagList<>();
        statData.getList().forEach(s -> tagList.add(new TagString(s.name())));
        itemProxy.editTag(tagCompound1 -> tagCompound1.put(getTagPath(), tagList));
    }

    @Override
    public @Nullable EnumListData<E> getLoadedTag(@NotNull ItemProxy itemProxy) {
        if (!itemProxy.has(getTagPath(), TagList.TYPE_ID)) {
            return new EnumListData<>(new ArrayList<>());
        }
        TagList<TagString> tagList = (TagList<TagString>) itemProxy.get(getTagPath());
        List<Enum> list = new ArrayList<>();
        for (int i = 0; i < tagList.size(); i++) {
            list.add(Enum.valueOf(clazz, tagList.get(i).getString()));
        }
        return new EnumListData(list);
    }

}
