package com.io.yutian.elementoriginlib.mongodb;

import com.mongodb.ConnectionString;
import com.mongodb.MongoClientSettings;
import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoDatabase;
import org.bson.UuidRepresentation;
import org.bson.codecs.UuidCodec;
import org.bson.codecs.configuration.CodecRegistries;
import org.bson.codecs.configuration.CodecRegistry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

public class MongoDBIO {

    private final static Logger LOG = LoggerFactory.getLogger(MongoDBIO.class);

    private static final CodecRegistry CODEC_REGISTRY = CodecRegistries.fromRegistries(
            CodecRegistries.fromCodecs(new UuidCodec(UuidRepresentation.STANDARD)),
            MongoClientSettings.getDefaultCodecRegistry()
    );

    private MongoDatabase mongoDatabase;

    public MongoDBIO(String host, int port, String username, String password, String dbname) {
        String connectionString = String.format(
                "mongodb://%s:%s@%s:%d/?authSource=admin",
                URLEncoder.encode(username, StandardCharsets.UTF_8),
                URLEncoder.encode(password, StandardCharsets.UTF_8),
                host, port
        );
        MongoClientSettings settings = MongoClientSettings.builder()
                .applyConnectionString(new ConnectionString(connectionString))
                .codecRegistry(CODEC_REGISTRY)
                .uuidRepresentation(UuidRepresentation.STANDARD)
                .build();
        MongoClient client = MongoClients.create(settings);
        mongoDatabase = client.getDatabase(dbname);
        if (mongoDatabase == null) {
            throw new IllegalArgumentException("获取数据库实例失败：" + dbname);
        }
    }

    public MongoDBIO(MongoClient mongo, String dbname) {
        mongoDatabase = mongo.getDatabase(dbname);
        if (mongoDatabase == null) {
            throw new IllegalArgumentException("获取数据库实例失败：" + dbname);
        }
    }

    public MongoDatabase getMongoDatabase() {
        return mongoDatabase;
    }

}
